@@ -13,9 +13,9 @@ BEGIN { $fields = {
 	"avs_result_code" => new WWW::Shopify::Field::String(),
 	"credit_card_bin" => new WWW::Shopify::Field::String(),
 	"cvv_result_code" => new WWW::Shopify::Field::String(),
-	"credit_card_numer" => new WWW::Shopify::Field::String('XXXX\-XXXX\-XXXX\-\d{4}'),
-	"credit_card_company" => new WWW::Shopify::Field::String("(Visa|Mastercard|AMEX")};
-}
+	"credit_card_number" => new WWW::Shopify::Field::String::Regex('XXXX-XXXX-XXXX-\d{4}'),
+	"credit_card_company" => new WWW::Shopify::Field::String()
+}; }
 sub plural($) { return $_[0]->singular(); }
 
 eval(__PACKAGE__->generate_accessors); die $@ if $@;
