@@ -13,7 +13,9 @@ BEGIN { $fields = {
 	"id" => new WWW::Shopify::Field::Identifier(),
 	"address1" => new WWW::Shopify::Field::String::Address(),
 	"city" => new WWW::Shopify::Field::String::City(),
-	"country" => new WWW::Shopify::Field::String::Country(),
+	"country" => new WWW::Shopify::Field::String("[A-Z]{3}"),
+	"country_name" => new WWW::Shopify::Field::String::Country(),
+	"country_code" => new WWW::Shopify::Field::String("[A-Z]{3}"),
 	"created_at" => new WWW::Shopify::Field::Date(min => '2010-01-01 00:00:00', max => 'now'),
 	"customer_email" => new WWW::Shopify::Field::String::Email(),
 	"domain" => new WWW::Shopify::Field::String::Hostname(),
@@ -23,11 +25,15 @@ BEGIN { $fields = {
 	"name" => new WWW::Shopify::Field::String::Words(1, 3),
 	"phone" => new WWW::Shopify::Field::String::Phone(),
 	"province" => new WWW::Shopify::Field::String::Words(1),
+	"province_code" => new WWW::Shopify::Field::String("[A-Z]{3}"),
 	"public" => new WWW::Shopify::Field::String(),
 	"source" => new WWW::Shopify::Field::String(),
+	"display_plan_name" => new WWW::Shopify::Field::String(),
 	"zip" => new WWW::Shopify::Field::String("[A-Z][0-9][A-Z] [0-9][A-Z][0-9]"),
 	"currency" => new WWW::Shopify::Field::Currency(),
 	"timezone" => new WWW::Shopify::Field::Timezone(),
+	"latitude" => new WWW::Shopify::Field::Float(),
+	"longitude" => new WWW::Shopify::Field::Float(),
 	"shop_owner" => new WWW::Shopify::Field::String::Name(),
 	"money_format" => new WWW::Shopify::Field::String("\$ \{\{amount\}\}"),
 	"money_with_currency_format" => new WWW::Shopify::Field::String("\$ \{\{amount\}\} USD"),
@@ -41,6 +47,7 @@ BEGIN { $fields = {
 sub creatable($) { return undef; }
 sub updatable($) { return undef; }
 sub deletable($) { return undef; }
+sub countable { return undef; }
 
 sub is_shop { return 1; }
 
