@@ -4,22 +4,27 @@ use strict;
 use warnings;
 
 package WWW::Shopify::URLHandler;
-use JSON qw(from_json to_json encode_json);
+use JSON qw(from_json to_json encode_json decode_json);
 use Data::Dumper;
 use Scalar::Util qw(weaken);
+use DateTime;
+use Encode;
 
 sub new {
 	my ($package, $parent) = @_;
-	my $self = bless {_parent => $parent}, $package;
+	my $self = bless {_parent => $parent, _default_headers => {}}, $package;
 	weaken($self->{_parent});
 	return $self;
 }
 sub parent { $_[0]->{_parent} = $_[1] if defined $_[1]; return $_[0]->{_parent}; }
 
-sub get_url($$@) {
+sub get_url {
 	my ($self, $url, $parameters) = @_;
 	my $uri = URI->new($url);
 	my %filtered = ();
+	if ($parameters->{fields} && ref($parameters->{fields}) && ref($parameters->{fields}) eq "ARRAY") {
+		$parameters->{fields} = join(",", @{$parameters->{fields}});
+	}
 	for (keys(%$parameters)) {
 		if ($parameters->{$_} && ref($parameters->{$_}) eq "DateTime") {
 			$filtered{$_} = $parameters->{$_}->strftime('%Y-%m-%d %H:%M:%S%z')
@@ -29,11 +34,10 @@ sub get_url($$@) {
 		}
 	}
 	$uri->query_form(\%filtered);
-	print STDERR "GET " . $uri->as_string . "\n" if $ENV{'SHOPIFY_LOG'};
 	my $request = HTTP::Request->new("GET", $uri);
 	$request->header("Accept" => "application/json");
+	$request->header($_ => $self->{_default_headers}->{$_}) for (keys(%{$self->{_default_headers}}));
 	my $response = $self->parent->ua->request($request);
-	print STDERR Dumper($response) if $ENV{'SHOPIFY_LOG'} && $ENV{'SHOPIFY_LOG'} > 1;
 	if (!$response->is_success) {
 		die new WWW::Shopify::Exception::CallLimit($response) if $response->code() == 429;
 		die new WWW::Shopify::Exception::InvalidKey($response) if $response->code() == 401;
@@ -47,18 +51,25 @@ sub get_url($$@) {
 	}
 	my $content = $response->decoded_content;
 	# From JSON because decodec content is already a perl internal string.
+	# Sigh. No. It's not. As per https://rt.cpan.org/Public/Bug/Display.html?id=82963; decoded_content doesn't actually do anything.
+	$content = decode("UTF-8", $content);
 	my $decoded = from_json($content);
 	return ($decoded, $response);
 }
 
-sub use_url($$$$) {
-	my ($self, $method, $url, $hash) = @_;
+use URI::Escape;
+
+sub use_url{
+	my ($self, $method, $url, $hash, $type) = @_;
+	$type = "application/json" unless $type;
 	my $request = HTTP::Request->new($method, $url);
-	print STDERR "$method $url\n" if $ENV{'SHOPIFY_LOG'};
-	$request->header("Accept" => "application/json", "Content-Type" => "application/json");
-	$request->content($hash ? encode_json($hash) : undef);
+	$request->header("Accept" => "application/json", "Content-Type" => $type);
+	if ($type =~ m/json/) {
+		$request->content($hash ? encode_json($hash) : undef);
+	} else {
+		$request->content($hash ? join("&", map { "$_=" . join(",", map { uri_escape($_) } ($hash->{$_} && ref($hash->{$_}) eq "ARRAY" ? @{$hash->{$_}} : ($hash->{$_}))) } keys(%$hash)) : undef);
+	}
 	my $response = $self->parent->ua->request($request);
-	print STDERR Dumper($response) if $ENV{'SHOPIFY_LOG'} && $ENV{'SHOPIFY_LOG'} > 1;
 	if (!$response->is_success) {
 		die new WWW::Shopify::Exception::CallLimit($response) if $response->code() == 429;
 		die new WWW::Shopify::Exception::InvalidKey($response) if $response->code() == 401;
@@ -69,12 +80,15 @@ sub use_url($$$$) {
 		die new WWW::Shopify::Exception("Unrecognized limit.") unless $limit =~ m/(\d+)\/\d+/;
 		$self->parent->api_calls($1);
 	}
+	my $content = $response->decoded_content;
 	# From JSON because decodec content is already a perl internal string.
-	my $decoded = (length($response->decoded_content) >= 2) ? from_json($response->decoded_content) : undef;
+	# Sigh. No. It's not. As per https://rt.cpan.org/Public/Bug/Display.html?id=82963; decoded_content doesn't actually do anything.
+	$content = decode("UTF-8", $content);
+	my $decoded = (length($content) >= 2) ? from_json($content) : undef;
 	return ($decoded, $response);
 }
-sub put_url($$$) { return shift->use_url("PUT", @_); }
-sub post_url($$$) { return shift->use_url("POST", @_); }
-sub delete_url($$$) { return shift->use_url("DELETE", @_); }
+sub put_url { return shift->use_url("PUT", @_); }
+sub post_url { return shift->use_url("POST", @_); }
+sub delete_url { return shift->use_url("DELETE", @_); }
 
 1;
